pl_lh=function(yt,xt,ytk,xtk,k){
  npow=1/3;
  beta=0.95;
  cz=-5;  
  
  #### Estimating variance covariance matrices for short horizon regression
  
  n1=length(xt);
  
  ydata=yt[1:(n1-1)];
  xdata=xt[1:(n1-1)];
  xdata2=xt[2:n1];
  dxdata=xdata2-xdata;

  n=length(xdata);

  Rnhat=0;
  RnhatCterm=0;
  zinstr=rep(0,n);

  ## OLS estimation of the 1st equation with constant: y{t}=x{t-1} ##
  
  olsreg1=lm(ydata~xdata);
  u0=as.vector(olsreg1$resid);
  Alpha1C=olsreg1$coef;
  Alpha1=Alpha1C[2];

  ## OLS estimation of the 2nd equation: x{t}=C*x{t-1}, C being diagonal ##
  
  olsreg2=lm(xdata2~xdata);
  Rnhat=olsreg2$coef[2];
  ux=as.vector(olsreg2$resid);
  
  ## var-cov  estimation ##
  ut=cbind(u0,ux);
  M1=n^npow;
  M=trunc(M1);
  vcovmat=lcov_mat1(x=ut,lam=M); # returns long, onesided sigma and lambda
  
  lambda=vcovmat$lambda;
  lambdax0=lambda[1,2]; #this is the transpose of MKS [(ky+1):p,1:ky];
  
  sigmaut=vcovmat$sigma;
  omegamatrix=vcovmat$long;
    
  omegaxx=omegamatrix[2,2];
  omega00b=sigmaut[1,1];
  omega0xb=sigmaut[1,2];
  omegafmd=omega00b-(omega0xb+lambdax0)*(omega0xb+lambdax0)/omegaxx;

  ## Long horizon regression estimation
  nk=n1-k;  
  ykdata=yt[(k):(n1-1)];
  
  ## Instrument
  
  dxdata=xt[-1]-xt[-n1];
  Rnz=1+cz/(nk^beta);
  zl1=c(0,filter(dxdata,Rnz,"rec"))[-n1];
  
  ## Computing Z(k) and X(k) 
  
  
  xkdata=na.omit(filter(xt[-n1],rep(1,k)))[1:nk];
  zkdata=na.omit(filter(zl1,rep(1,k)))[1:nk];
  
  ## Adjusting the sample size of the instrumet and demeaning
  
  zl1=zl1[1:nk];
  
  ykbar=mean(ykdata);
  xkbar=mean(xkdata);
  xbar=mean(xdata);
  zl1bar=mean(zkdata); # this not the mean of zl1 but the mean of zkdata, I use this notation (althoug a bit confusing) to be consistent with the notation used in the SH script
  
  ykdatadm=ykdata-ykbar;
  xkdatadm=xkdata-xkbar;
  xdatadm=xdata-xbar;   

  ## A ~ matrix calculation ##
 
  Alphatilb=sum(ykdatadm*zkdata)/sum(xkdatadm*zkdata);
   
  ## Wald Statistic
  
  
  M=sum(xkdatadm*zkdata)*sum(xkdatadm*zkdata)/sum(zkdata*zkdata);
  QH=omega00b/M;
  #MM=sum(zkdata*zkdata)*omega00b-nk*zl1bar*zl1bar*omegafmd;
  #QH=MM/(sum(zl1*xkdatadm)^2); 
  Wn3= Alphatilb*Alphatilb/QH;
  pval=1-pchisq(Wn3,1);
  return(list(Aivx=Alphatilb,WaldIVX=Wn3,pval=pval));   
}

lcov_mat1 <- function(x,lam){
	x <- as.matrix(x);
	nr <- NROW(x);
	nc <- NCOL(x);
	# alternative: lam <- floor(4*(nr/100)^(2/9)); # truncation parameter, see Newey West 1987
	for(i in 0:lam){
		gamma <- (1-i/(lam+1))/nr*t(x[1:(nr-i),])%*%x[(1+i):nr,]; # Bartlett window
		if(i==0){
			long <- gamma; # Omega
			onesided <- gamma; # Delta in KMS notation
			lambda <- 0;
			sigma <- gamma;
		}else {
			long <- long+gamma+t(gamma);
			onesided <- onesided+gamma;
			lambda <- lambda+gamma;
		} 
	}
	return(list(long=long, onesided=onesided, lambda=lambda, sigma=sigma));
}

