# Clear memory, set working directory and source IVX code
rm(list=ls(all=T));
setwd("../"); # need to set working directory
source("./R_functions/PhillipsLee_LH.R");

##################################################################################
############ Baumeister and Kilian (BK) Expectations Data ########################
#### Consensus Economics data are proprietary and are not reported ###############
#### Column 1: Date for the Spot Price 
#### Column 2: Date for Expected Spot Price, 1 month ahead
#### Column 3: Date for Expected Spot Price, 3 months ahead
#### Column 4: Date for Expected Spot Price, 12 months ahead
#### Column 5: Spot Price from Bloomberg
#### Column 6: BK 3m forecast
#### Column 7: BK 12m forecast
#### Column 8: Future Spot Price, 3 months ahead 
#### Column 9: Future Spot Price, 12 months ahead 
#### Column 10: 3 month Forecast Error (3m Future Spot Price - 3m BK Forecast)
#### Column 11: 12 month Forecast Error (12m Future Spot Price - 12m BK Forecast)
#### Column 12: Future Spot Price, 1 month ahead 
#### Column 13: BK 1m forecast  
##########################################################################################################

bkData=read.table("./Data/bk_data.txt",header=T,sep="\t");

#### Results 
ivx_results=matrix(,2,1); # rows correspond to 3 and 12 month results; columns to financial market expectations 
colnames(ivx_results)=c("Financial Market Expectations" );
rownames(ivx_results)=c("3 months","12 months");

#### Prepare regression variables for Fama-type regression. Horizon n=3

yt=as.vector(bkData[,12],"numeric")-as.vector(bkData[,13],"numeric");   # S[t+1]-E(S[t+1])
xt=as.vector(bkData[,13],"numeric")-as.vector(bkData[,5],"numeric");   # E(S[t+1])-S[t]
ytk=as.vector(bkData[,8],"numeric")-as.vector(bkData[,6],"numeric");   # S[t+n]-E(S[t+n])
xtk=as.vector(bkData[,6],"numeric")-as.vector(bkData[,5],"numeric");   # E(S[t+n])-S[t]

n=3;
T=nrow(bkData);
end=T-n;
minsize=floor(T*(0.01+1.8/sqrt(T)));  
rep=end-minsize+1;
results_ivx=results_pl=matrix(,rep,3);
results_ls=matrix(,rep,3);
k=n;
for(i in 1:rep){
  aux_pl=pl_lh(yt=yt[i:(minsize+i+n-1)],xt=xt[i:(minsize+i+n-1)],ytk=ytk[i:(minsize+i-1)],xtk=xtk[i:(minsize+i-1)],k=n);
  results_pl[i,]=c(aux_pl$Aivx,aux_pl$WaldIVX,aux_pl$pval);
}

pl_res=cbind(results_pl,ifelse(results_pl[,3]<0.1 & results_pl[,1]>0,1,0))
ztest=c(rep(NA,minsize),sqrt(results_pl[,2])*sign(results_pl[,1]));
maxZ=max(na.omit(ztest));
print(paste("Financial Market Expectations 3m -------------","Max IVX:",round(maxZ,3)," Crit Value:",round(qnorm(1-.05/rep),3),sep=" "));

ivx_results[1,1]=maxZ;

#### Prepare regression variables for Fama-type regression. Horizon n=12

yt=as.vector(bkData[,12],"numeric")-as.vector(bkData[,13],"numeric");   # S[t+1]-E(S[t+1])
xt=as.vector(bkData[,13],"numeric")-as.vector(bkData[,5],"numeric");   # E(S[t+1])-S[t]
ytk=as.vector(bkData[,9],"numeric")-as.vector(bkData[,7],"numeric");   # S[t+n]-E(S[t+n])
xtk=as.vector(bkData[,7],"numeric")-as.vector(bkData[,5],"numeric");   # E(S[t+n])-S[t]

n=12;
T=nrow(bkData);
end=T-n;
minsize=floor(T*(0.01+1.8/sqrt(T)));  
rep=end-minsize+1;
results_ivx=results_pl=matrix(,rep,3);
results_ls=matrix(,rep,3);
k=n;
for(i in 1:rep){
  aux_pl=pl_lh(yt=yt[i:(minsize+i+n-1)],xt=xt[i:(minsize+i+n-1)],ytk=ytk[i:(minsize+i-1)],xtk=xtk[i:(minsize+i-1)],k=n);
  results_pl[i,]=c(aux_pl$Aivx,aux_pl$WaldIVX,aux_pl$pval);
}

pl_res=cbind(results_pl,ifelse(results_pl[,3]<0.1 & results_pl[,1]>0,1,0))
ztest=c(rep(NA,minsize),sqrt(results_pl[,2])*sign(results_pl[,1]));
maxZ=max(na.omit(ztest));
print(paste("Financial Market Expectations 12m -------------","Max IVX:",round(maxZ,3)," Crit Value:",round(qnorm(1-.05/rep),3),sep=" "));
ivx_results[2,1]=maxZ;

